/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/MessageList.java,v 1.10 2004/03/27 21:34:10 dgraham Exp $
 * $Revision: 1.10 $
 * $Date: 2004/03/27 21:34:10 $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources;

import org.apache.commons.resources.Message;

import java.util.Iterator;

/**
 * <p>
 * A class that encapsulates messages. MessageList can be either global
 * or they are specific to a particular bean property.
 * </p>
 * <p>
 * Each individual message is described by an <code>Message</code>
 * object, which contains a message key (to be looked up in an appropriate
 * message resources database), and up to four placeholder arguments used for
 * parametric substitution in the resulting message.
 * </p>
 * <p>
 * <strong>IMPLEMENTATION NOTE</strong> - It is assumed that these objects
 * are created and manipulated only within the context of a single thread.
 * Therefore, no synchronization is required for access to internal
 * collections.
 * </p>
 * Orginally based on org.apache.http.action.ActionMessages, Revision 1.7.
 */
public interface MessageList {

    /**
     * A default key to represent "global" messages
     * that do not pertain to a particular property.
     */
    public static final String GLOBAL_MESSAGE_KEY =
            "org.apache.commons.resources.GLOBAL_MESSAGE";

    /**
     * @return The default global message key
     */
    public String getGlobalMessageKey();

    /**
     * @param globalMessageKey The new default global message key
     */
    public void setGlobalMessageKey(String globalMessageKey);

    /**
     * Add a message to the set of messages for the specified property.  An
     * order of the property/key is maintained based on the initial addition
     * of the property/key.
     *
     * @param property  Property name (or MessageList.GLOBAL_MESSAGE_KEY)
     * @param message   The message to be added
     */
    public void add(String property, Message message);

    /**
     * Add a message to the set of messages for the "global" property.  An
     * order of the property/key is maintained based on the initial addition
     * of the property/key.
     *
     * @param message   The message to be added
     */
    public void add(Message message);

    /**
     * Adds the messages from the given <code>MessageList</code> object to
     * this set of messages. The messages are added in the order they are returned from
     * the properties() method. If a message's property is already in the current
     * <code>MessageList</code> object it is added to the end of the list for that
     * property. If a message's property is not in the current list it is added to the end
     * of the properties.
     *
     * @param messages The <code>MessageList</code> object to be added.
     */
    public void add(MessageList messages);

    /**
     * Clear all messages recorded by this object.
     */
    public void clear();
    
    /**
     * Determines if the MessageList's messages have been accessed one or more
     * times.  Returns <code>true</code> if the <code>get()</code> or 
     * <code>get(String)</code> methods are called.  
     * @return <code>true</code> if the messages have been accessed one or more
     * times.
     */
    public boolean isAccessed();

    /**
     * @return Return <code>true</code> if there are no messages recorded
     * in this collection, or <code>false</code> otherwise.
     */
    public boolean isEmpty();

    /**
     * Return the set of all recorded messages, without distinction
     * by which property the messages are associated with.  If there are
     * no messages recorded, an empty enumeration is returned.
     */
    public Iterator get();

    /**
     * Return the set of messages related to a specific property.
     * If there are no such messages, an empty enumeration is returned.
     *
     * @param property Property name (or ActionMessages.GLOBAL_MESSAGE_KEY)
     */
    public Iterator get(String property);

    /**
     * Return the set of property names for which at least one message has
     * been recorded.  If there are no messages, an empty Iterator is returned.
     * If you have recorded global messages, the String value of
     * <code>MessageList.GLOBAL_MESSAGE</code> will be one of the returned
     * property names.
     */
    public Iterator properties();

    /**
     * Return the number of messages recorded for all properties (including
     * global messages).  <strong>NOTE</strong> - it is more efficient to call
     * <code>isEmpty()</code> if all you care about is whether or not there are
     * any messages at all.
     */
    public int size();

    /**
     * Return the number of messages associated with the specified property.
     *
     * @param property Property name (or MessageList.GLOBAL_MESSAGE_KEY
     */
    public int size(String property);

}

