/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/impl/ResourcesBase.java,v 1.11 2004/03/27 21:34:09 dgraham Exp $
 * $Revision: 1.11 $
 * $Date: 2004/03/27 21:34:09 $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.Iterator;
import java.util.Locale;
import java.util.TimeZone;

import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;

/**
 * <p>Convenience base class for 
 * {@link org.apache.commons.resources.Resources} implementations.</p>
 *
 * <p>Default implementations of the content retrieval methods are provided
 * for all methods except <code>getObject()</code>.  The default methods for
 * the other content retrieval methods are coded in terms of
 * <code>getString()</code>, on the assumption that most uses of
 * {@link org.apache.commons.resources.Resources} are of this type.  
 * However, they can be easily overridden as needed.</p>
 */
public abstract class ResourcesBase implements Resources {


    // ----------------------------------------------------------- Constructors


    /**
     * <p>Create a new {@link org.apache.commons.resources.Resources} instance 
     * with no name.</p>
     */
    public ResourcesBase() {

        this(null);

    }


    /**
     * <p>Create a new {@link org.apache.commons.resources.Resources} instance 
     * with the specified logical name.</p>
     *
     * @param name Logical name of the new instance
     */
    public ResourcesBase(String name) {

        this.name = name;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The logical name of this {@link org.apache.commons.resources.Resources} 
     * instance.</p>
     */
    private String name = null;


    /**
     * <p>Flag indicating whether resource getter methods should return
     * <code>null</code> (instead of throwing an exception) on invalid
     * key values.</p>
     */
    private boolean returnNull = true;


    // ------------------------------------------------------ Lifecycle Methods


    /**
     * <p>This must be called to initialize the data content of this
     * {@link org.apache.commons.resources.Resources} instance, before 
     * any of the <code>getXxx()</code> methods are called.</p>
     *
     * <p>The default implementation does nothing.</p>
     *
     * @exception ResourcesException if an error occurs during initialization
     */
    public void init() throws ResourcesException {

        ; // The default implementation does nothing

    }


    /**
     * <p>This method must be called when the manager of this resource
     * decides that it's no longer needed.  After this method is called,
     * no further calls to any of the <code>getXxx()</code> methods are
     * allowed.</p>
     *
     * <p>The default implementation does nothing.</p>
     *
     * @exception ResourcesException if an error occurs during finalization
     */
    public void destroy() throws ResourcesException {

        ; // The default implementation does nothing

    }


    // ------------------------------------------------------------- Properties


    public abstract Iterator getKeys();


    /**
     * <p>Return the logical name of this {@link org.apache.commons.resources.Resources} 
     * instance.</p>
     */
    public String getName() {

        return (this.name);

    }


    /**
     * <p>Return <code>true</code> if resource getter methods will return
     * <code>null</code> instead of throwing an exception on invalid
     * key values.</p>
     */
    public boolean isReturnNull() {

        return (this.returnNull);

    }


    /**
     * <p>Set a flag determining whether resource getter methods should
     * return <code>null</code> instead of throwing an exception on
     * invalid key values.</p>
     *
     * @param returnNull The new flag value
     */
    public void setReturnNull(boolean returnNull) {

        this.returnNull = returnNull;

    }


    // ---------------------------------------------- Content Retrieval Methods


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * byte array, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getString()</code> and
     * converts the value to a byte array.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public byte[] getBytes(String key, Locale locale, TimeZone timeZone) {

        String value = getString(key, locale, timeZone);
        if (value == null) {
            return (null);
        } else {
            return (value.getBytes());
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * InputStream, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getsBytes()</code>
     * and returns an input stream over the resulting byte array.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public InputStream getInputStream(String key, Locale locale,
                                      TimeZone timeZone) {

        byte bytes[] = getBytes(key, locale, timeZone);
        if (bytes == null) {
            bytes = new byte[0];
        }
        return (new ByteArrayInputStream(bytes));

    }


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * Object, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>There is no default implementation of this method.  Concrete
     * subclasses must provide such an implementation.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public abstract Object getObject(String key, Locale locale, TimeZone timeZone);


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * Reader, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getString()</code>
     * and returns a reader over the resulting characters.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public Reader getReader(String key, Locale locale, TimeZone timeZone) {

        String value = getString(key, locale, timeZone);
        if (value == null) {
            return (null);
        } else {
            return (new StringReader(value));
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * String, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * <p>The default implementation calls <code>getObject()</code>
     * and converts the result to a String if necessary.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public String getString(String key, Locale locale, TimeZone timeZone) {

        Object value = getObject(key, locale, timeZone);
        if (value == null) {
            return ((String) null);
        } else if (value instanceof String) {
            return ((String) value);
        } else {
            return (value.toString());
        }

    }

}
