/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/impl/ResourcesFactoryBase.java,v 1.9 2004/03/27 21:34:09 dgraham Exp $
 * $Revision: 1.9 $
 * $Date: 2004/03/27 21:34:09 $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;
import org.apache.commons.resources.ResourcesFactory;

/**
 * <p>Convenience base class for {@link ResourcesFactory} implementations.
 * This implementation caches the {@link Resources} instances returned by
 * a protected <code>createResources()</code> method, which must be implemented
 * by concrete subclasses.</p>
 */
public abstract class ResourcesFactoryBase implements ResourcesFactory {
    

    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The set of {@link Resources} instances previously created by
     * this {@link ResourcesFactory}, keyed by logical name.</p>
     */
    protected Map resources = new HashMap();


    /**
     * <p>The <code>returnNull</code> property value that will be configured
     * on {@link Resources} instances created by this factory.</p>
     */
    protected boolean returnNull = true;


    // ------------------------------------------------------------- Properties


    /**
     * <p>Return the <code>returnNull</code> property value that will be
     * configured on {@link Resources} instances created by this factory.</p>
     */
    public boolean isReturnNull() {

        return (this.returnNull);

    }


    /**
     * <p>Set the <code>returnNull</code> property value that will be
     * configured on {@link Resources} instances created by this factory.</p>
     *
     * @param returnNull The new value to delegate
     */
    public void setReturnNull(boolean returnNull) {

        this.returnNull = returnNull;

    }


    // --------------------------------------------------------- Public Methods


    /**
     * <p>Create (if necessary) and return a {@link Resources} instance
     * for the specified logical name, with a default configuration.
     * The default implementation of this method treats the name as the
     * configuration String as well, and calls the
     * <code>getResources(String,String)</code> method.
     *
     * @param name Logical name of the {@link Resources} instance to
     *  be returned
     *
     * @exception ResourcesException if a {@link Resources} instance
     *  of the specified logical name cannot be returned.
     */
    public Resources getResources(String name)
        throws ResourcesException {

        return (getResources(name, name));

    }


    /**
     * <p>Create (if necessary) and return a {@link Resources} instance
     * for the specified logical name, with a configuration based on
     * the specified configuration String.</p>
     *
     * @param name Logical name of the {@link Resources} instance to
     *  be returned
     * @param config Configuration string for this resource (meaning
     *  is dependent upon the {@link ResourcesFactory} implementation
     *  being utilized), or <code>null</code> for the default
     *  configuration
     *
     * @exception ResourcesException if a {@link Resources} instance
     *  of the specified logical name cannot be returned.
     */
    public Resources getResources(String name, String config)
        throws ResourcesException {

        synchronized (resources) {
            Resources instance = (Resources) resources.get(name);
            if (instance == null) {
                instance = createResources(name, config);
                resources.put(name, instance);
            }
            return (instance);
        }

    }


    /**
     * <p>Release any internal references to {@link Resources} instances
     * that have been returned previously, after calling the
     * <code>destroy()</code> method on each such instance.</p>
     *
     * @exception ResourcesException if a problem occurred while releasing
     */
    public void release() throws ResourcesException {

        synchronized (resources) {
            Iterator names = resources.keySet().iterator();
            while (names.hasNext()) {
                String name = (String) names.next();
                ((Resources) resources.get(name)).destroy();
            }
            resources.clear();
        }

    }



    // ------------------------------------------------------ Protected Methods


    /**
     * <p>Create and return a new {@link Resources} instance with the
     * specified logical name, after calling its <code>init()</code>
     * method and delegating the relevant properties.  Concrete
     * subclasses <strong>MUST</strong> implement this method.</p>
     *
     * @param name Logical name of the {@link Resources} instance to create
     * @param config Configuration string for this resource (if any)
     *
     * @exception ResourcesException if a {@link Resources} instance
     *  of the specified logical name cannot be created.
     */
    protected abstract Resources createResources(String name, String config)
        throws ResourcesException;


}
