/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/impl/WebappPropertyResources.java,v 1.10 2004/03/27 21:34:09 dgraham Exp $
 * $Revision: 1.10 $
 * $Date: 2004/03/27 21:34:09 $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * <p>Concrete implementation of {@link org.apache.commons.resources.Resources} that wraps a family
 * (one per <code>Locale</code> of properties files that share a base
 * context-relative path for servlet context resources, and have
 * name suffixes reflecting the <code>Locale</code> for which
 * the document's messages apply.  Resources are looked up in a hierarchy
 * of properties files in a manner identical to that performed by
 * <code>java.util.ResourceBundle.getBundle().</code>.</p>
 *
 * <p>The base resource path passed to our constructor must contain the
 * context-relative base name of the properties file family.
 * For example, if the base path is passed as
 * <code>http://localhost/foo/Bar</code>, the resources for the
 * <code>en_US</code> Locale would be stored under URL
 * <code>http://localhost/foo/Bar_en_US.properties</code>, and the default
 * resources would be stored in
 * <code>http://localhost/foo/Bar.properties</code>.</p>
 */
public class WebappPropertyResources extends CollectionResourcesBase {

    /**
     * <p>The <code>Log</code> instance for this class.</p>
     */
	private static final Log log =
		LogFactory.getLog(WebappPropertyResources.class);

    // ----------------------------------------------------------- Constructors


    /**
     * <p>Create a new {@link org.apache.commons.resources.Resources} instance with the specified
     * logical name and base resource URL.</p>
     *
     * @param name Logical name of the new instance
     * @param base Base URL of the family of properties files that contain
     *  the resource keys and values
     * @param servletContext the <code>ServletContext</code> instance
     *  to use for resolving resource references
     */
    public WebappPropertyResources(String name, String base,
                                   ServletContext servletContext) {

        super(name, base);
        this.servletContext = servletContext;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The <code>ServletContext</code> instance for resolving
     * our resources references.</p>
     */
    private ServletContext servletContext = null;


    // ------------------------------------------------------ Protected Methods


    /**
     * <p>Return a <code>Map</code> containing the name-value mappings for
     * the specified base URL and requested <code>Locale</code>, if there
     * are any.  If there are no defined mappings for the specified
     * <code>Locale</code>, return an empty <code>Map</code> instead.</p>
     *
     * <p>Concrete subclasses must override this method to perform the
     * appropriate lookup.  A typical implementation will construct an
     * absolute URL based on the specified base URL and <code>Locale</code>,
     * retrieve the specified resource file (if any), and parse it into
     * a <code>Map</code> structure.</p>
     *
     * <p>Caching of previously retrieved <code>Map</code>s (if any) should
     * be performed by callers of this method.  Therefore, this method should
     * always attempt to retrieve the specified resource and load it
     * appropriately.</p>
     *
     * @param base Base URL of the resource files for this {@link org.apache.commons.resources.Resources}
     *  instance
     * @param locale <code>Locale</code> for which name-value mappings
     *  are requested
     */
    protected Map getLocaleMap(String baseUrl, Locale locale) {

        if (log.isDebugEnabled()) {
            log.debug("Loading locale '" + locale + "' resources from base '" +
                    baseUrl + "'");
        }

        Properties props = new Properties();
        String name = baseUrl + getLocaleSuffix(locale) + ".properties";
        InputStream stream = null;

        try {

            // Open an input stream to the URL for this locale (if any)
            if (log.isTraceEnabled()) {
                log.trace("Complete path is '" + name + "'");
            }
            stream = servletContext.getResourceAsStream(name);

            // Parse the input stream and populate the name-value mappings map
            if (stream != null) {
                if (log.isTraceEnabled()) {
                    log.trace("Parsing input resource");
                }
                props.load(stream);
            }

        } catch (FileNotFoundException e) {

            // Log and swallow this exception
            if (log.isDebugEnabled()) {
                log.debug("No resources for locale '" + locale +
                          "' from base '" + baseUrl + "'");
            }
            props.clear();

        } catch (IOException e) {

            log.warn("IOException loading locale '" + locale +
                     "' from base '" + baseUrl + "'", e);
            props.clear();

        } finally {

            // Close the input stream that was opened earlier
            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException e) {
                    log.error("Error closing stream.", e);
                }
                stream = null;
            }

        }

        // Return the populated (or empty) properties
        return (props);

    }


}
